﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using EAS_Protocol.Helpers;
using EAS_Protocol.Models;
using EAS_Protocol;
using System.IO;
using System.Xml;
using System.Text;
using System.Xml.XPath;

namespace EAS_Web_API.Controllers
{
    public class CalendarController : ApiController
    {
        [HttpGet]
        [AllowAnonymous]
        /// <summary>
        /// Syncs all contacts in the "Default Contacts" folder
        /// </summary>
        public List<CalendarEntry> syncCalendar(string serverAddress, string credentials, string DeviceID, string DeviceType)
        {       
            //The algorithm for syncing the calendar is a three-step process:
            // - Do a FolderSync to get the Ids for all folders, and identify the folder for Default contacts
            // - Do a Sync with a SyncKey == 0 to get a new SyncKey
            // - Do a Sync with the new SyncKey

            List<CalendarEntry> calendar = new List<CalendarEntry>();

            string strUsername = HelperMethods.getUsername(credentials);
            string strDeviceID = DeviceID;
            string strDeviceType = DeviceType;
            string creds = credentials;

            string easUrl = serverAddress;

            string syncKey = "0";
            string collectionId = "";

            string ASVersion;
            string UserAgent;

            //Filter out necessary values from headers
            //Fall back to defaults if not provided
            //To do : Should defaults be provided or should the API throw an error?
            var headerValues = new List<string>().AsEnumerable();
            if (Request.Headers.TryGetValues("MS-ASProtocolVersion", out headerValues))
            {
                ASVersion = headerValues.First().ToString();
            }
            else
            {
                ASVersion = "14.1";
            }
            if (Request.Headers.TryGetValues("DeviceUserAgent", out headerValues))
            {
                UserAgent = headerValues.First().ToString();
            }
            else
            {
                UserAgent = "EASWeb";
            }


            Device dev = new Device();
            dev.ASVersion = ASVersion;
            dev.DeviceID = DeviceID;
            dev.DeviceType = DeviceType;
            dev.UserAgent = UserAgent;
            dev.Username = strUsername;
            dev.ServerAddress = easUrl;
            dev.Credentials = credentials;

            string fsResponse = AS_CMD.FolderSync(dev);
            collectionId = AS_CMD.GetCollectionID(fsResponse, FolderSyncTypes.Default_Calendar);
            syncKey = AS_CMD.Sync(dev, collectionId);
            string syncResponse = AS_CMD.Sync(dev, syncKey, collectionId); 

            try
            {
                //We parse out the contact elements
                var xmlSync = new XmlDocument();
                xmlSync.LoadXml(syncResponse);

                //Check the status of the sync request
                var status = xmlSync.GetElementsByTagName("Status");

                //Status of "1" == Success
                if (status[0].InnerXml == "1")
                {
                    var contactResults = xmlSync.GetElementsByTagName("ApplicationData");
                    int contactItems = contactResults.Count;

                    #region                    
                    for (int i = 0; i < contactItems; i++)
                    {                        
                        calendar.Add(new CalendarEntry());
                        //A given calendar may have empty/null properties 
                        //These tags will not be included in the XML
                        //so we need to check the count for each xmlItem before adding the value.
                        //Note that the default is to truncate the entries meaning that not all properties are included
                        //This means there are properties in the model having a null value
                        var xmlItem = xmlSync.GetElementsByTagName("TimeZone");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].TimeZone = xmlItem[i].InnerXml;                            
                        }
                        xmlItem = xmlSync.GetElementsByTagName("AllDayEvent");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].AllDayEvent = xmlItem[i].InnerXml;
                        }
                        
                        xmlItem = xmlSync.GetElementsByTagName("BusyStatus");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].BusyStatus = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("DTStamp");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].DTStamp = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("EndTime");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].EndTime = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Location");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].Location = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("MeetingStatus");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].MeetingStatus = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Organizer_Email");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].Organizer_Email = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Organizer_Name");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].Organizer_Name = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Type");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].Type = xmlItem[i].InnerXml;
                        }                        
                        xmlItem = xmlSync.GetElementsByTagName("Reminder");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].Reminder = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Sensitivity");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].Sensitivity = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Subject");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].Subject = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("StartTime");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].StartTime = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("UID");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].UID = xmlItem[i].InnerXml;
                        }                        
                        xmlItem = xmlSync.GetElementsByTagName("ResponseRequested");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].ResponseRequested = xmlItem[i].InnerXml;
                        }                        
                        xmlItem = xmlSync.GetElementsByTagName("ResponseType");
                        if (xmlItem[i] != null)
                        {
                            calendar[i].ResponseType = xmlItem[i].InnerXml;
                        }                                                
                    }
                    #endregion

                }
                //Status !1 == !OK
                else
                {
                    switch (status[0].InnerXml)
                    {
                        case "2":
                            break;

                        default:
                            break;
                    }
                }
            }
            catch (Exception)
            {
            }            
            //#endregion

            return calendar;
        }
        
    }
}
