﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using EAS_Protocol.Helpers;
using EAS_Protocol.Models;
using EAS_Protocol;
using System.IO;
using System.Xml;
using System.Text;
using System.Xml.XPath;

namespace EAS_Web_API.Controllers
{
    public class ContactsController : ApiController
    {
        [HttpGet]
        [AllowAnonymous]
        /// <summary>
        /// Syncs all contacts in the "Default Contacts" folder
        /// </summary>
        public List<Contact> syncContacts(string serverAddress, string credentials, string DeviceID, string DeviceType)
        {       
            //The algorithm for syncing all contacts is a three-step process:
            // - Do a FolderSync to get the Ids for all folders, and identify the folder for Default contacts
            // - Do a Sync with a SyncKey == 0 to get a new SyncKey
            // - Do a Sync with the new SyncKey

            List<Contact> contacts = new List<Contact>();

            string strUsername = HelperMethods.getUsername(credentials);
            string strDeviceID = DeviceID;
            string strDeviceType = DeviceType;
            string creds = credentials;

            string easUrl = serverAddress;

            string syncKey = "0";
            string collectionId = "";

            string ASVersion;
            string UserAgent;

            //Filter out necessary values from headers
            //Fall back to defaults if not provided
            //To do : Should defaults be provided or should the API throw an error?
            var headerValues = new List<string>().AsEnumerable();
            if (Request.Headers.TryGetValues("MS-ASProtocolVersion", out headerValues))
            {
                ASVersion = headerValues.First().ToString();
            }
            else
            {
                ASVersion = "14.1";
            }
            if (Request.Headers.TryGetValues("DeviceUserAgent", out headerValues))
            {
                UserAgent = headerValues.First().ToString();
            }
            else
            {
                UserAgent = "EASWeb";
            }


            Device dev = new Device();
            dev.ASVersion = ASVersion;
            dev.DeviceID = DeviceID;
            dev.DeviceType = DeviceType;
            dev.UserAgent = UserAgent;
            dev.Username = strUsername;
            dev.ServerAddress = easUrl;
            dev.Credentials = credentials;

            string fsResponse = AS_CMD.FolderSync(dev);
            collectionId = AS_CMD.GetCollectionID(fsResponse, FolderSyncTypes.Default_Contacts);
            syncKey = AS_CMD.Sync(dev, collectionId);
            string syncResponse = AS_CMD.Sync(dev, syncKey, collectionId);            

            try
            {
                //We parse out the contact elements
                var xmlSync = new XmlDocument();
                xmlSync.LoadXml(syncResponse);

                //Check the status of the sync request
                var status = xmlSync.GetElementsByTagName("Status");

                //Status of "1" == Success
                if (status[0].InnerXml == "1")
                {
                    var contactResults = xmlSync.GetElementsByTagName("ApplicationData");
                    int contactItems = contactResults.Count;

                    #region                    
                    for (int i = 0; i < contactItems; i++)
                    {                        
                        contacts.Add(new Contact());
                        //A given contact may have empty/null properties 
                        //These tags will not be included in the XML
                        //so we need to check the count for each xmlItem before adding the value.
                        var xmlItem = xmlSync.GetElementsByTagName("AssistantName");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].AssistantName = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("AssistantPhoneNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].AssistantPhoneNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Birthday");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Birthday = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Business2PhoneNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Business2PhoneNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("BusinessAddressCity");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].BusinessAddressCity = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("BusinessAddressCountry");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].BusinessAddressCountry = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("BusinessAddressPostalCode");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].BusinessAddressPostalCode = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("BusinessAddressState");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].BusinessAddressState = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("BusinessAddressStreet");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].BusinessAddressStreet = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("BusinessFaxNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].BusinessFaxNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("BusinessPhoneNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].BusinessPhoneNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("CarPhoneNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].CarPhoneNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Categories");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Categories = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Category");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Category = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Children");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Children = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Child");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Child = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("CompanyName");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].CompanyName = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Department");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Department = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Email1Address");
                        if (xmlItem[i] != null)
                        {
                            //We're using InnerText for EmailAddress to get "plain" formatting
                            //InnerXml returns HTML-encoded text
                            contacts[i].Email1Address = xmlItem[i].InnerText;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Email2Address");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Email2Address = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Email3Address");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Email3Address = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("FileAs");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].FileAs = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Home2PhoneNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Home2PhoneNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("HomeAddressCity");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].HomeAddressCity = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("HomeAddressCountry");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].HomeAddressCountry = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("HomeAddressPostalCode");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].HomeAddressPostalCode = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("HomeAddressState");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].HomeAddressState = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("HomeAddressStreet");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].HomeAddressStreet = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("HomeFaxNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].HomeFaxNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("HomePhoneNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].HomePhoneNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("JobTitle");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].JobTitle = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("LastName");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].LastName = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("MiddleName");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].MiddleName = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("MobilePhoneNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].MobilePhoneNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("OfficeLocation");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].OfficeLocation = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("OtherAddressCity");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].OtherAddressCity = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("OtherAddressCountry");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].OtherAddressCountry = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("OtherAddressPostalCode");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].OtherAddressPostalCode = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("OtherAddressState");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].OtherAddressState = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("OtherAddressStreet");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].OtherAddressStreet = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("PagerNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].PagerNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("RadioPhoneNumber");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].RadioPhoneNumber = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Spouse");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Spouse = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Suffix");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Suffix = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Title");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Title = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("WebPage");
                        if (xmlItem.Count > 0)
                        {
                            contacts[i].WebPage = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("YomiCompanyName");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].YomiCompanyName = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("YomiFirstName");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].YomiFirstName = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("YomiLastName");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].YomiLastName = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Picture");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Picture = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Alias");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].Alias = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("WeightedRank");
                        if (xmlItem[i] != null)
                        {
                            contacts[i].WeightedRank = xmlItem[i].InnerXml;
                        }
                    }
                    //#endregion

                }
                //Status !1 == !OK
                else
                {
                    switch (status[0].InnerXml)
                    {
                        case "2":
                            break;

                        default:
                            break;
                    }
                }
            }
            catch (Exception)
            {
            }            
            #endregion
            return contacts;
        }
        
    }
}
