﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.IO;
using System.Xml;
using System.Text;
using System.Threading.Tasks;
using EAS_Protocol;
using EAS_Protocol.Models;
using EAS_Protocol.Helpers;
using System.Configuration;

namespace EASWebAccess.Controllers
{
    public class GALController : ApiController
    {
        [HttpGet]
        [AllowAnonymous]
        public List<GALEntry> queryGAL(string searchFor, int range, string serverAddress, string credentials, string DeviceID, string DeviceType)
        {            
            List<GALEntry> galItems = new List<GALEntry>();            
            
            string strUsername = HelperMethods.getUsername(credentials);
            //string strDeviceID = DeviceID;
            //string strDeviceType = DeviceType;            

            string easUrl = serverAddress;           

            string ASVersion;
            string UserAgent;

            //Filter out necessary values from headers
            //Fall back to defaults if not provided
            //To do : Should defaults be provided or should the API throw an error?
            var headerValues = new List<string>().AsEnumerable();
            if (Request.Headers.TryGetValues("MS-ASProtocolVersion", out headerValues))
            {
                ASVersion = headerValues.First().ToString();
            }
            else
            {
                ASVersion = "14.1";
            }
            if (Request.Headers.TryGetValues("DeviceUserAgent", out headerValues))
            {
                UserAgent = headerValues.First().ToString();
            }
            else
            {
                UserAgent = "EASWeb";
            }

            Device dev = new Device();
            dev.ASVersion = ASVersion;
            dev.DeviceID = DeviceID;
            dev.DeviceType = DeviceType;
            dev.UserAgent = UserAgent;
            dev.Username = strUsername;
            dev.ServerAddress = easUrl;
            dev.Credentials = credentials;

            string searchResponse = AS_CMD.Search(dev,SearchTypes.SEARCH_GAL,searchFor,0,1);           

            try
            {
                var xmlGalEntry = new XmlDocument();
                xmlGalEntry.LoadXml(searchResponse);

                //Check the status of the search request
                var status = xmlGalEntry.GetElementsByTagName("Status");

                //Status of "1" == Success
                if (status[0].InnerXml == "1")
                {
                    //The "Total" tag will include the actual number of hits the search yielded
                    var totalResults = xmlGalEntry.GetElementsByTagName("Total");
                    string totalItems = totalResults[0].InnerXml;

                    //The "Result" tag will contain the number of entries as indicated by the "Range" parameter in the search request
                    var filteredResults = xmlGalEntry.GetElementsByTagName("Result");
                    int filteredItems = filteredResults.Count;

                    //int numberOfItems;
                    //Int32.TryParse(filteredItems, out numberOfItems);

                    for (int i =0; i<filteredItems;i++)
                    {
                        galItems.Add(new GALEntry());

                        //A given GAL entry may have empty/null properties in AD
                        //These tags will not be included in the XML
                        //so we need to check the count for each xmlItem before adding the value.
                        var xmlItem = xmlGalEntry.GetElementsByTagName("DisplayName");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].DisplayName = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("Phone");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].Phone = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("Office");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].Office = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("Title");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].Title = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("Company");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].Company = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("Alias");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].Alias = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("FirstName");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].FirstName = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("LastName");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].LastName = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("HomePhone");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].HomePhone = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("MobilePhone");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].MobilePhone = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("EmailAddress");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].EmailAddress = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("Picture");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].Picture = xmlItem[i].InnerXml;
                        }

                        xmlItem = xmlGalEntry.GetElementsByTagName("Data");
                        if (xmlItem.Count > 0)
                        {
                            galItems[i].Data = xmlItem[i].InnerXml;
                        }
                    }
                }
                //Status !1 == !OK
                else                
                {
                    switch (status[0].InnerXml)
                    {
                        case "2":                            
                            break;

                        default:
                            break;
                    }
                }
            }
            catch (Exception)
            {
             
            }


            return galItems;
        }

    }
}
