﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using EAS_Protocol.Helpers;
using EAS_Protocol.Models;
using EAS_Protocol;
using System.IO;
using System.Xml;
using System.Text;
using System.Xml.XPath;

namespace EAS_Web_API.Controllers
{
    public class TasksController : ApiController
    {
        [HttpGet]
        [AllowAnonymous]
        /// <summary>
        /// Syncs all Notes in the "Default Notes" folder
        /// </summary>
        public List<Task> syncTasks(string serverAddress, string credentials, string DeviceID, string DeviceType)
        {
            //The algorithm for syncing tasks is a three-step process:
            // - Do a FolderSync to get the Ids for all folders, and identify the folder for Default tasks
            // - Do a Sync with a SyncKey == 0 to get a new SyncKey
            // - Do a Sync with the new SyncKey

            List<Task> tasks = new List<Task>();

            string strUsername = HelperMethods.getUsername(credentials);
            string strDeviceID = DeviceID;
            string strDeviceType = DeviceType;
            string creds = credentials;

            string easUrl = serverAddress;

            string syncKey = "0";
            string collectionId = "";

            string ASVersion;
            string UserAgent;

            //Filter out necessary values from headers
            //Fall back to defaults if not provided
            //To do : Should defaults be provided or should the API throw an error?
            var headerValues = new List<string>().AsEnumerable();
            if (Request.Headers.TryGetValues("MS-ASProtocolVersion", out headerValues))
            {
                ASVersion = headerValues.First().ToString();
            }
            else
            {
                ASVersion = "14.1";
            }
            if (Request.Headers.TryGetValues("DeviceUserAgent", out headerValues))
            {
                UserAgent = headerValues.First().ToString();
            }
            else
            {
                UserAgent = "EASWeb";
            }

            Device dev = new Device();
            dev.ASVersion = ASVersion;
            dev.DeviceID = DeviceID;
            dev.DeviceType = DeviceType;
            dev.UserAgent = UserAgent;
            dev.Username = strUsername;
            dev.ServerAddress = easUrl;
            dev.Credentials = credentials;            
            
            string fsResponse = AS_CMD.FolderSync(dev);
            collectionId = AS_CMD.GetCollectionID(fsResponse, FolderSyncTypes.Default_Tasks);
            syncKey = AS_CMD.Sync(dev,collectionId);
            string syncResponse = AS_CMD.Sync(dev, syncKey, collectionId);            

            try
            {
                //We parse out the task elements
                var xmlSync = new XmlDocument();
                xmlSync.LoadXml(syncResponse);

                //Check the status of the sync request
                var status = xmlSync.GetElementsByTagName("Status");

                //Status of "1" == Success
                if (status[0].InnerXml == "1")
                {
                    var taskResults = xmlSync.GetElementsByTagName("ApplicationData");
                    int taskItems = taskResults.Count;

                    #region
                    for (int i = 0; i < taskItems; i++)
                    {
                        tasks.Add(new Task());
                        //A given task may have empty/null properties 
                        //These tags will not be included in the XML
                        //so we need to check the count for each xmlItem before adding the value.
                        //Note that the default is to truncate the entries meaning that not all properties are included
                        //This means there are properties in the model having a null value
                        var xmlItem = xmlSync.GetElementsByTagName("Category");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Category = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Complete");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Complete = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("DateCompleted");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].DateCompleted = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("DueDate");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].DueDate = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("UTCDueDate");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].UTCDueDate = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Importance");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Importance = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Recurrence");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Recurrence = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Type");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Type = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Start");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Start = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Until");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Until = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Occurrences");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Occurrences = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Interval");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Interval = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("DayOfMonth");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].DayOfMonth = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("DayOfWeek");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].DayOfWeek = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("WeekOfMonth");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].WeekOfMonth = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("MonthOfYear");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].MonthOfYear = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Regenerate");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Regenerate = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("DeadOccur");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].DeadOccur = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("ReminderSet");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].ReminderSet = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("ReminderTime");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].ReminderTime = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Sensitivity");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Sensitivity = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("StartDate");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].StartDate = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("UTCStartDate");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].UTCStartDate = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("Subject");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].Subject = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("OrdinalDate");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].OrdinalDate = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("SubOrdinalDate");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].SubOrdinalDate = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("CalendarType");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].CalendarType = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("IsLeapMonth");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].IsLeapMonth = xmlItem[i].InnerXml;
                        }
                        xmlItem = xmlSync.GetElementsByTagName("FirstDayOfWeek");
                        if (xmlItem[i] != null)
                        {
                            tasks[i].FirstDayOfWeek = xmlItem[i].InnerXml;
                        }
                    }
                    #endregion

                }
                //Status !1 == !OK
                else
                {
                    switch (status[0].InnerXml)
                    {
                        case "2":
                            break;

                        default:
                            break;
                    }
                }
            }
            catch (Exception)
            {
            }            

            return tasks;
        }
    }
}

